@@ -22,4 +22,5 @@ from .github import DownloadGitHubFile
 from .great_expectations import RunGreatExpectationsValidation
 from .sqlite import SQLiteInsert, SQLiteSQLtoDF, SQLiteQuery
 from .supermetrics import SupermetricsToCSV, SupermetricsToDF
-from .cloud_for_customers import CloudForCustomersToCSV, CloudForCustomersToDF
+from .sharepoint import SharepointToDF
+from .cloud_for_customers import c4c_report_to_df, c4c_to_df
